@@ -3,6 +3,7 @@
 
 # imports.
 from dev0s.classes.defaults.exceptions import Exceptions
+from dev0s.classes import utils
 from dev0s.classes import console
 from dev0s.classes.defaults.files import *
 from dev0s.classes.defaults.defaults import defaults
@@ -24,8 +25,21 @@ class Version(object):
 	def __init__(self, 
 		# the version value (#1).
 		value="1.0.00",
+		# the file path (optional) (str, FilePath).
+		path=None,
+		# the default value (opional) (str)
+		default=None,
+		# load the specified path data on initialization (bool).
+		load=False,
 	):
 
+		# docs.
+		DOCS = {
+			"module":"Version", 
+			"initialized":False,
+			"description":[], 
+			"chapter": "Code", }
+
 		# defaults.
 		#self.__class__.__name__ = "Version"
 
@@ -37,7 +51,25 @@ class Version(object):
 		self.value = str(value).replace(" ","").replace("\n","").replace("\r","")
 		self.int = int(str(self.value).replace(".",""))
 
+		# path.
+		if path == False: self.file_path = self.fp = None # used in local memory (not fysical)
+		else: self.file_path = self.fp = Formats.FilePath(path)
+		if default != None and not Files.exists(self.file_path.path): self.save(array=default)
+		if load: self.load()
+
 		#
+	def save(self, value=None, path=None, sudo=False):
+		if value == None: value = self.value
+		if path == None: path = self.file_path.path
+		utils.__check_memory_only__(path)
+		self.assign(string)
+		return Files.save(path, str(string), format="str", sudo=sudo)
+	def load(self, default=None, sudo=False):
+		utils.__check_memory_only__(self.file_path.path)
+		if not os.path.exists(self.file_path.path) and default != None: 
+			self.save(default, sudo=sudo)
+		self.assign(Files.load(self.file_path.path, format="str", sudo=sudo))
+		return self.value
 	def increase(self, value=None, count=1):
 		if value == None: value = self.value
 
@@ -162,21 +194,23 @@ class Version(object):
 	def __eq__(self, version):
 		if isinstance(version, None.__class__):
 			return False
-		elif not isinstance(version, self.__class__):
-			raise Exceptions.FormatError(f"Can not compare object {self.__class__} & {version.__class__}.")
-		return self.int == version.int
+		elif isinstance(version, self.__class__):
+			return self.int == version.int
+		else:
+			return str(self.int) == str(version)
 	def __ne__(self, version):
 		if isinstance(version, None.__class__):
-			return False
-		elif not isinstance(version, self.__class__):
-			raise Exceptions.FormatError(f"Can not compare object {self.__class__} & {version.__class__}.")
-		return self.int != version.int
+			return True
+		elif isinstance(version, self.__class__):
+			return self.int != version.int
+		else:
+			return str(self.int) != str(version)
 	
 	# support 'in' operator.
 	def __contains__(self, string):
 		if isinstance(string, (list, Files.Array)):
 			for i in string:
-				if i in str(self.value):
+				if str(i) in str(self.value):
 					return True
 			return False
 		else:
@@ -213,7 +247,7 @@ class Version(object):
 			value = value.value
 		elif not isinstance(value, self.__class__):
 			raise Exceptions.FormatError(f"Can not assign object {self.__class__} & {value.__class__}.")
-		self.value = value
+		self.value = str(value)
 		self.int = int(str(self.value).replace(".",""))
 		return self
 	# return raw data.
@@ -236,6 +270,13 @@ class Script(Files.File):
 		default=None,
 	):
 		
+		# docs.
+		DOCS = {
+			"module":"Script", 
+			"initialized":False,
+			"description":[], 
+			"chapter": "Code", }
+
 		# check self instance.
 		if isinstance(data, (Files.File, Script)):
 			data = data.data
@@ -298,6 +339,14 @@ class Python(Files.File):
 		# the default data (create if path does not exist).
 		default=None,
 	):
+
+		# docs.
+		DOCS = {
+			"module":"Python", 
+			"initialized":False,
+			"description":[], 
+			"chapter": "Code", }
+
 		# check self instance.
 		if isinstance(data, (Files.File, Script, Python)):
 			data = data.data
@@ -315,7 +364,7 @@ class Python(Files.File):
 		self.executable = executable
 		
 		#
-
+		#
 	# execute.
 	def execute(self, 
 		# the script executable (leave None to use defult self.executable) (#1).
@@ -328,7 +377,7 @@ class Python(Files.File):
 		return execute(path=data, executable=executable)
 
 		#
-		
+		#
 	# slice classes.
 	def slice_classes(self,
 		# the data (str).
@@ -352,8 +401,12 @@ class Python(Files.File):
 		# only for self recursive use.
 		keep_indent=False,
 	):
+
+		# checks.
 		if data == None: data = self.data
 		if data == None: raise Exceptions.InvalidUsage("Define parameter: [data].")
+
+		# normalize.
 		data = data.replace("	","    ")
 		before = "\n"
 		if len(data) > 0 and data[0] != before: data = before+str(data)
@@ -382,6 +435,8 @@ class Python(Files.File):
 			elif "DOCS    = " in data: data = data.replace("DOCS    = ", "DOCS = ")
 			elif "DOCS     = " in data: data = data.replace("DOCS     = ", "DOCS = ")
 			else: break
+
+		# vars.
 		classes = []
 		to_slice = []
 		for i in range(100):
@@ -398,6 +453,8 @@ class Python(Files.File):
 		elif len(slices) > 1:
 			slices = slices[1:]
 		slice_count = 0
+
+		# slice.
 		for class_ in slices:
 			if class_ not in ["", " "]:
 				try: previous = slices[slice_count-1]
@@ -521,7 +578,45 @@ class Python(Files.File):
 								"type":"class",
 							})
 			slice_count += 1
+
+		# fill DOCS customization.
+		_classes_ = []
+		for info in classes:
+			initialized, module, description, chapter = False, None, [], None
+			if len(info["functions"]) > 0:
+				for func in info["functions"]:
+					if "__init__" in func["name"]:
+						test = "DOCS = {"
+						if test in func["code"]:
+							customization = ast.literal_eval(str(String(func["code"].split(test[:-1])[1]).slice_dict(depth=1)))
+							if "initialized" in customization:
+								info["initialized"] = customization["initialized"]
+							if "module" in customization:
+								info["module"] = customization["module"]
+							if "notes" in customization:
+								info["description"] = customization["description"]
+							if "chapter" in customization:
+								info["chapter"] = customization["chapter"]
+							#if "return" in customization:
+							#	info["return"] = customization["return"]
+							break
+			for key,value in {
+				"initialized":False,
+				"module":None,
+				"description":[],
+				"chapter":None,
+				#"return":"_",
+			}.items():
+				try:info[key]
+				except: info[key] = value
+			_classes_.append(info)
+		classes = _classes_
+
+		# handler.
 		return classes
+
+		#
+
 	# slice functions.
 	def slice_functions(self,
 		# the data (str).
@@ -636,6 +731,7 @@ class Python(Files.File):
 			if name in banned_functions: return False
 			if not system_functions and "__init__" not in name and len(name) >= 4 and name[:2] == "__" and name[-2:] == "__": return False
 			return True
+			#
 
 		# vars.
 		functions = []
@@ -750,7 +846,7 @@ class Python(Files.File):
 								full_inside_class = f"{i['init_line'].split('(')[0]}{i['parameters']}:\n{i['raw_code']}"
 								class_code = class_code.replace("):  \n","):\n").replace("):   \n","):\n").replace("):    \n","):\n").replace("):     \n","):\n").replace("):	\n","):\n").replace(full_inside_class, "")
 						return_ = None
-						if " return Response" in str(raw_class_code) or "\nreturn Response" in str(raw_class_code):
+						if (" return response" in str(raw_class_code) or "\nreturn response" in str(raw_class_code)) or (" return dev0s.response." in str(raw_class_code) or "\nreturn dev0s.response." in str(raw_class_code)) or (" return _response_." in str(raw_class_code) or "\nreturn _response_." in str(raw_class_code)):
 							return_ = "response"
 						elif " return " not in str(raw_class_code) or "\nreturn " not in str(raw_class_code):
 							return_ = "_"
@@ -793,7 +889,43 @@ class Python(Files.File):
 							"return":return_,
 						})
 			slice_count += 1
+
+		# fill DOCS customization.
+		_functions_ = []
+		for info in functions:
+			initialized, module, description, chapter = False, None, [], None
+			if "__init__" not in info["name"]:
+				test = "DOCS = {"
+				if test in info["code"]:
+					customization = ast.literal_eval(str(String(info["code"].split(test[:-1])[1]).slice_dict(depth=1)))
+					if "initialized" in customization:
+						info["initialized"] = customization["initialized"]
+					if "module" in customization:
+						info["module"] = customization["module"]
+					if "description" in customization:
+						info["description"] = customization["description"]
+					if "chapter" in customization:
+						info["chapter"] = customization["chapter"]
+					if "return" in customization:
+						info["return"] = customization["return"]
+					break
+			for key,value in {
+				"initialized":False,
+				"module":None,
+				"description":[],
+				"chapter":None,
+				"return":"_",
+			}.items():
+				try:info[key]
+				except: info[key] = value
+			_functions_.append(info)
+		functions = _functions_
+
+		# handler.
 		return functions
+
+		#
+
 	# build code examples.
 	def build_code_examples(self,
 		# the path (leave None to use self.fp.path).
@@ -853,43 +985,30 @@ class Python(Files.File):
 			""" """
 
 			# fill DOCS customization.
-			initialized, module, description = False, None, None
-			if len(info["functions"]) > 0:
-				for func in info["functions"]:
-					if "__init__" in func["name"]:
-						set = False
-						test = "DOCS = {"
-						if test in func["code"]:
-							customization = ast.literal_eval(str(String(func["code"].split(test[:-1])[1]).slice_dict(depth=1)))
-							if "initialized" in customization:
-								initialized = customization["initialized"]
-							if "module" in customization:
-								module = customization["module"]
-							if "notes" in customization:
-								description = customization["description"]
-							break
+			initialized, module, description = info["initialized"], info["module"], info["description"]
 			if module == None:
 				module = info["name"]
 
-
-				#if [initialized, module, notes] != [None, None, None]:
-				#	initialized_name = module
+			# docs.
 			parameters = clean_params(info["parameters"]).replace("self.", initialized_name+".")
 			if initialized == True:
-				doc = (
-					"\n\n# import the " + str(module) + " object class." + "\n" +
-					"")
+				doc = "\n"
 				if description != None:
-					for i in description: doc += f"{i}\n"
+					for i in description: doc += f"{i}\n".replace("</attribute>", "").replace("<attribute>", "").replace("</parameter>", "").replace("<parameter>", "")
 				doc += (
-					f"from {package} import " + module.split('.')[0] + "\n" +
+					"\n# import the " + str(module) + " object class." + "\n" +
+					"")
+				doc += (
+					#f"from {package} import " + module.split('.')[0] + "\n" +
+					f"import {package}" + "\n" +
 					"")
 			else:
-				doc = (
-					"\n\n# initialize the " + str(module) + " object class." + "\n" +
-					"")
+				doc = "\n"
 				if description != None:
-					for i in description: doc += f"{i}\n"
+					for i in description: doc += f"{i}\n".replace("</attribute>", "").replace("<attribute>", "").replace("</parameter>", "").replace("<parameter>", "")
+				doc += (
+					"\n# initialize the " + str(module) + " object class." + "\n" +
+					"")
 				doc += (
 					str(initialized_name)+" = " + module + str(parameters) + "\n" +
 					"")
@@ -968,9 +1087,12 @@ class Python(Files.File):
 				"```" + "\n"
 				"")
 			return doc
+			#
 
 		# slice.
 		if path == None: path = self.fp.path
+		if path == None:
+			raise dev0s.exceptions.InvalidUsage(f"Define parameter [path] (str).")
 		if gfp.exists(path=path) and gfp.directory(path=path):
 			path = Directory(path).paths(extensions=["py"], banned=banned, banned_names=banned_names, banned_basenames=banned_basenames, recursive=True)
 		if isinstance(path, (str, String)): path = [path]
@@ -993,16 +1115,29 @@ class Python(Files.File):
 			# iterate.
 			for info in first_indent_functions:
 				if "__init__" not in info["name"]:
-					if info["raw_name"] in list(docs_dict.keys()):
+					try: info["chapter"]
+					except KeyError: info["chapter"] = "Functions"
+					if info["chapter"] == None: info["chapter"] = "Functions"
+					try: docs_dict[info["chapter"]]
+					except KeyError: docs_dict[info["chapter"]] = {}
+					if info["raw_name"] in list(docs_dict[info["chapter"]].keys()):
 						try:info["raw_name"] = info["raw_name"].split("-")[0]+"-"+str(int(info["raw_name"].split("-")[1])+1)
 						except: info["raw_name"] = info["raw_name"]+"-1"
-					else:
-						docs_dict[info["raw_name"]] = {
-							"docs":build_func_doc(info),
-							"raw_name":info["raw_name"].split("-")[0],
-							"unique_name":info["raw_name"],
-							"type":info["type"],
-						}
+					docs_dict[info["chapter"]][info["raw_name"]] = {
+						"docs":build_func_doc(info),
+						"name":info["name"],
+						"raw_name":info["raw_name"].split("-")[0],
+						"unique_name":info["raw_name"],
+						"type":info["type"],
+						#
+						"parameters":info["parameters"],
+						"initialized":info["initialized"],
+						"module":info["module"],
+						"description":info["description"],
+						"chapter":info["chapter"],
+						"return":info["return"],
+						"path":_path_,
+					}
 			for info in classes:
 				l_banned = False
 				for i in banned_class_types:
@@ -1010,17 +1145,34 @@ class Python(Files.File):
 						l_banned = True
 						break
 				if not l_banned:
-					if info["raw_name"] in list(docs_dict.keys()):
+					try: info["chapter"]
+					except KeyError: info["chapter"] = "Classes"
+					if info["chapter"] == None: info["chapter"] = "Classes"
+					try: docs_dict[info["chapter"]]
+					except KeyError: docs_dict[info["chapter"]] = {}
+					if info["raw_name"] in list(docs_dict[info["chapter"]].keys()):
 						try:info["raw_name"] = info["raw_name"].split("-")[0]+"-"+str(int(info["raw_name"].split("-")[1])+1)
 						except: info["raw_name"] = info["raw_name"]+"-1"
-					else:
-						docs_dict[info["raw_name"]] = {
-							"docs":build_class_doc(info),
-							"raw_name":info["raw_name"].split("-")[0],
-							"unique_name":info["raw_name"],
-							"type":info["type"],
-							"functions":info["functions"],
-						}
+					functions = []
+					for func in info["functions"]:
+						func["path"] = _path_
+						functions.append(func)
+					info["functions"] = functions
+					docs_dict[info["chapter"]][info["raw_name"]] = {
+						"docs":build_class_doc(info),
+						"name":info["name"],
+						"raw_name":info["raw_name"].split("-")[0],
+						"unique_name":info["raw_name"],
+						"type":info["type"],
+						"functions":info["functions"],
+						#
+						"parameters":info["parameters"],
+						"initialized":info["initialized"],
+						"module":info["module"],
+						"description":info["description"],
+						"chapter":info["chapter"],
+						"path":_path_,
+					}
 			if log_level >= 0: loader.stop()
 		
 		# sort alphabetically.
@@ -1029,29 +1181,32 @@ class Python(Files.File):
 		# assign unique id's.
 		_docs_dict_ = {}
 		all_ids = []
-		for _, info in docs_dict.items():
-			while True:
-				if info["raw_name"] in all_ids:
-					try:info["raw_name"] = info["raw_name"].split("-")[0]+"-"+str(int(info["raw_name"].split("-")[1])+1)
-					except: info["raw_name"] = info["raw_name"]+"-1"
-				else:
-					all_ids.append(info["raw_name"])
-					if info["type"] == "class":
-						new = []
-						for _info_ in info["functions"]:
-							if "__init__" not in _info_["name"]:
-								while True:
-									if _info_["raw_name"] in all_ids:
-										#print(_info_["name"], "vs", all_ids)
-										try:_info_["raw_name"] = _info_["raw_name"].split("-")[0]+"-"+str(int(_info_["raw_name"].split("-")[1])+1)
-										except: _info_["raw_name"] = _info_["raw_name"]+"-1"
-									else:
-										all_ids.append(_info_["raw_name"]) 
-										break
-								new.append(_info_)
-						info["functions"] = new
-					_docs_dict_[info["raw_name"]] = info
-					break
+		for chapter, l_docs_dict in docs_dict.items():
+			try: _docs_dict_[chapter]
+			except KeyError: _docs_dict_[chapter] = {}
+			for _, info in l_docs_dict.items():
+				while True:
+					if info["raw_name"] in all_ids:
+						try:info["raw_name"] = info["raw_name"].split("-")[0]+"-"+str(int(info["raw_name"].split("-")[1])+1)
+						except: info["raw_name"] = info["raw_name"]+"-1"
+					else:
+						all_ids.append(info["raw_name"])
+						if info["type"] == "class":
+							new = []
+							for _info_ in info["functions"]:
+								if "__init__" not in _info_["name"]:
+									while True:
+										if _info_["raw_name"] in all_ids:
+											#print(_info_["name"], "vs", all_ids)
+											try:_info_["raw_name"] = _info_["raw_name"].split("-")[0]+"-"+str(int(_info_["raw_name"].split("-")[1])+1)
+											except: _info_["raw_name"] = _info_["raw_name"]+"-1"
+										else:
+											all_ids.append(_info_["raw_name"]) 
+											break
+									new.append(_info_)
+							info["functions"] = new
+						_docs_dict_[chapter][info["raw_name"]] = info
+						break
 		docs_dict = _docs_dict_
 
 		# return.
@@ -1065,6 +1220,9 @@ class Python(Files.File):
 				return docs_dict, all_ids
 			else:
 				return docs_dict
+
+		#
+
 	# build readme code examples.
 	def build_readme(self,
 		# the package name.
@@ -1155,40 +1313,49 @@ class Python(Files.File):
 			l = (
 				f"""\n# Code Examples:""" + "\n" + 
 				"")
+			
 			# add table content.
 			l += f"""\n### Table of content:""" + "\n"
-			for id, example in code_examples.items(): 
-				if example["type"] == "class":
-					l += f"- [__{example['raw_name']}__](#{example['unique_name'].replace(' ','-').lower()})" + "\n"
-					properties = False
-					for fexample in example["functions"]:
-						if "__init__" not in fexample["raw_name"]: 
-							if fexample["type"] in ["property"]:
-								if not properties:
-									properties = True
-									unique = "properties"
-									while True:
-										if unique not in all_ids:
-											all_ids.append(unique)
-											break
-										else:
-											try:unique = unique.split("-")[0]+"-"+str(int(unique.split("-")[1])+1)
-											except: unique = unique+"-1"
-									unique_name = str(unique)
+			last_chapter = None
+			for chapter, l_code_examples in code_examples.items():
+				for id, example in l_code_examples.items(): 
+					if example["type"] == "class":
+						if chapter == None: chapter = "Classes"
+						if chapter != last_chapter:
+							l += f"- [__{chapter}__](#{chapter.replace(' ','-').lower()})" + "\n"
+							last_chapter = chapter
+						l += f"  - [__{example['raw_name']}__](#{example['unique_name'].replace(' ','-').lower()})" + "\n"
+						properties = False
+						for fexample in example["functions"]:
+							if "__init__" not in fexample["raw_name"]: 
+								if fexample["type"] in ["property"]:
+									if not properties:
+										properties = True
+										unique = "properties"
+										while True:
+											if unique not in all_ids:
+												all_ids.append(unique)
+												break
+											else:
+												try:unique = unique.split("-")[0]+"-"+str(int(unique.split("-")[1])+1)
+												except: unique = unique+"-1"
+										unique_name = str(unique)
+										raw_name = str(fexample['raw_name']).split("-")[0]
+										l += f"    * [{raw_name}](#{unique_name.replace(' ','-').lower()})" + "\n"	
+								else:
+									unique_name = str(fexample['raw_name'])
 									raw_name = str(fexample['raw_name']).split("-")[0]
-									l += f"  * [{raw_name}](#{unique_name.replace(' ','-').lower()})" + "\n"	
-							else:
-								unique_name = str(fexample['raw_name'])
-								raw_name = str(fexample['raw_name']).split("-")[0]
-								l += f"  * [{raw_name}](#{unique_name.replace(' ','-').lower()})" + "\n"
+									l += f"    * [{raw_name}](#{unique_name.replace(' ','-').lower()})" + "\n"
 			l += "\n"#------\n\n"
 			_code_examples_ += l
 			_readme_ += l
+			
 			# add codes.
-			for _, example in code_examples.items(): 
-				#print(example)
-				_code_examples_ += example["docs"]
-				_readme_ += example["docs"]
+			for chapter, l_code_examples in code_examples.items():
+				for _, example in l_code_examples.items(): 
+					#print(example)
+					_code_examples_ += example["docs"]
+					_readme_ += example["docs"]
 
 		# create header table of content.
 		code_open, headers = False, []
@@ -1228,15 +1395,88 @@ class Python(Files.File):
 		# examples export.
 		if examples != None:
 			if log_level >= 0: loader = console.Loader(f"Saving created code examples to [{examples}]")
-			if not os.path.exists(gfp.base(path=examples)): os.system(f"mkdir -p {gfp.base(path=examples)}")
+			if not Files.exists(gfp.base(examples)): Files.create(gfp.base(examples), directory=True)
 			Files.save(examples, _code_examples_)
 			if log_level >= 0: loader.stop()
 
 		# handler.
-		return _readme_
+		return code_examples, _readme_
+		
 		#
 
-	#
+	# clean python code.
+	def clean(self,
+		# forced enabled ignored the warning prompt (bool).
+		forced=False,
+		# the self path (str) (leave None to use self.path).
+		path=None,
+	):
+
+		raise ValueError("Coming soon.")
+
+		# prompt warning.
+		if not forced and not console.input(f"&RED&Warning&END&: You are cleaning python code [{path}], do you wish to proceed?", yes_no=True):
+			raise Exceptions.AbortError("User aborted.")
+
+		# vars.
+		if path == None: path = self.fp.path
+		if path == None: raise Exceptions.InvalidUsage("<Python.clean> define parameter: [path].")
+
+		""" load data """
+		data = Files.load(path=path, format="str")
+
+		""" normalize data """
+		while True:
+			if "	" in data: data = data.replace("	", "    ")
+			else: break
+
+		"""
+		add # to the the end of each indentation.
+		"""
+		def check_indentation_end(data):
+			new_data, string = "", String()
+			items = String(data).iterate_lines()
+			max = len(items)-1
+			last_indent = None
+			last_chars = {
+				"1":"",
+				"blank":False,
+			}
+			for linecount, line in items:
+				indent = string.line_indent(line=line)
+				print(f"{indent}:[{line}]")
+				if last_indent != None and last_chars["1"] != "#" and indent < last_indent:
+					print("Found lower indent difference")
+					difference = indent - last_indent
+					new_data += string.indent(indent=last_indent)+"#\n"
+				if linecount == max: new_data += line
+				else: new_data += line+"\n"
+				last_indent = indent
+				if len(line) > 0: l = line[len(line)-1]
+				else: l = ""
+
+				# add last chars.
+				last_chars["blank"] = line in [""]
+				for key, _ in last_chars.items():
+					if key not in ["blank"]:
+						last_chars[key] += l
+						while True:
+							if len(last_chars[key]) >= int(key): last_chars[key] = last_chars[key][:-1]
+							else: break
+
+				#
+
+			return new_data
+
+		# call cleanings.
+		old = str(data)
+		data = check_indentation_end(data)
+
+		# results.
+		print("\n=====================================\nFrom:\n"+old)
+		print("\n=====================================\nTo:\n"+data)
+
+
 	# object instance.
 	def instance(self):
 		return "Python"
@@ -1246,7 +1486,6 @@ class Python(Files.File):
 		
 		#
 
-	#
 	# support self assignment.
 	def assign(self, string):
 		if isinstance(data, (int, float)):
