@@ -20,7 +20,7 @@ class Requests(object):
 			"module":"dev0s.requests", 
 			"initialized":True,
 			"description":[], 
-			"chapter":"requests",
+			"chapter":"Requests",
 		}
 
 		# attributes.
@@ -43,6 +43,8 @@ class Requests(object):
 				value = str(_response_.serialize(value.raw(), safe=True))
 			elif isinstance(value, (list,Array,dict,Dictionary)):
 				value = str(value)
+			else:
+				value = Formats.denitialize(value)
 			_data_[key] = value
 		return _data_
 
@@ -101,7 +103,7 @@ class Requests(object):
 				"module":"dev0s.requests.RestAPI", 
 				"initialized":False,
 				"description":[], 
-				"chapter":"requests",
+				"chapter":"Requests",
 			}
 
 			# defaults.
@@ -134,6 +136,4 @@ class Requests(object):
 # initialized classes.
 requests = Requests()
 
-#
-
 #
\ No newline at end of file
